/*
  Created:  2012.08.16 for Arduino
   Author:  James Buck, VE3BUX
      Web:  http://www.ve3bux.com
  If you use this code and / or the associated library, all I ask is that you "give a nod" by making a small mention
  in your code's documentation section. I've worked hard on getting this working, so a little recognition is appreciated. :)

  Modified : 2020.04.18 CAT-FT857-ESP32 V1.0
  Author   : Ph Lonc, F6CZV
  Port on ESP32 board TTGO T-Display
  Inclusion of a color TFT screen on SPI bus to display :
  - frequency and mode,
  - RX/TX status,
  - S Meter value,
  - VFO A / B status,
  - Keyer and Break-in status if mode is CW or CWR,
  - DSP configuration,
  - SPLIT status.
  The VFO status, Keyer, Break-in, DSP configutation and SPLIT status are read from the radio EEPROM.
  SoftwareSerial was replaced by HardwareSerial available for ESP32. ESP32 has 3 UARTs (0 to 2) but only UART 2 (Serial2) is usable without problem. 
  As on development boards pin 16 and 17 dedicated by default to Serial2 are either not shown or are used for other purpose 
  it is possible to affect pin 25, 26 and 27 for Rx/tx. 
  As it is a graphical TFT screen screen must be cleared before any write to avoid scories of previous writes.
  The tft.fillRect(x,y,width,height, color) function was used for that.

LIMITATION OF LIABILITY :
 This source code is provided "as-is". It may contain bugs. 
Any damages resulting from its use is done under the sole responsibilty of the user/developper and beyond my responsibility.
  
  ------------------

  FT857D Display

  This example sketch implements various commands of the FT-857D library to display the parameters of a Yaesu FT-857D radio.
  Attach your radio's CAT interface to the following ESP32 TTGO T-Display pins to begin:

    Radio CAT port GND -> T-Display GND
    Radio CAT port TX  -> T-Display pin 27 
    Radio CAT port RX  -> T-Display pin 26 

  If once you compile and upload the code, you are unable to see someting on the screen try reversing
  pins 26 and 27. It may be necessary to power down the radio and to reset the MCU prior to powering the radio back on.

*/
#include <TFT_eSPI.h> // F6CZV ESP 32
#include <SPI.h> // F6CZV ESP32
#include <HardwareSerial.h> // replace softwareserial for Arduino
#include <Wire.h> // F6CZV
#include "FT857D-ESP32.h"     // 
#ifndef TFT_DISPOFF
#define TFT_DISPOFF 0x28
#endif

#ifndef TFT_SLPIN
#define TFT_SLPIN   0x10
#endif

#define TFT_MOSI            19
#define TFT_SCLK            18
#define TFT_CS              5
#define TFT_DC              16
#define TFT_RST             23

#define TFT_BL          4  // Display backlight control pin
#define ADC_EN          14
#define ADC_PIN         34
#define BUTTON_1        35 // Not used
#define BUTTON_2        0  // Not used

TFT_eSPI tft = TFT_eSPI(135, 240); // Invoke custom library

int btnDisp = true;

FT857D radio;           // define "radio" so that we may pass CAT commands
bool On_Air = false;
bool Split = false;
bool Keyer;
bool Break_In;
byte MeterConf;
char VFO;
char prevVFO = ' ';
bool DNF = false;
bool DNR = false;
bool DBF = false;
bool AGC = false;
String blank = "      ";
char * radmode = "   ";
char * prevmode = "   ";
String Smeter;
String prevSmeter = "     ";
int dly = 500;            // delay for x milliseconds between commands 

void setup() {

/*---- Working on a way to allow easier user input of I/O pins for software serial emulation
  ---- Right now, it is necessary to edit the FT857D.cpp file and to edit the line which states:
  ---- hardwareSerial begin (baud,8N2,26,27); // rx,tx
  ---- change the rx & tx pins to values which suit your application and save the FT857D.cpp file
*/

  tft.init();
  tft.setRotation(1);
  tft.fillScreen(TFT_BLUE);
  tft.setTextColor(TFT_WHITE);
  tft.setTextSize(3);

  if (TFT_BL > 0) { // TFT_BL has been set in the TFT_eSPI library in the User Setup file TTGO_T_Display.h
    pinMode(TFT_BL, OUTPUT); // Set backlight pin to output mode
    digitalWrite(TFT_BL, TFT_BACKLIGHT_ON); // Turn backlight on. TFT_BACKLIGHT_ON has been set in the TFT_eSPI library in the User Setup file TTGO_T_Display.h
  }



 Serial.begin(4800);
 radio.begin(4800);    // as with Serial.begin(4800); we wish to start the hardware serial port
                        // so that we may control a radio via CAT commands
  tft.setTextSize(3);
  tft.setCursor(0, 35); // (x colonne , y ligne) - F6CZV
  tft.print("VFO"); // F6CZV
 
}

void loop() {         
 
/* perform a frequency sweep
   for (long freq=0; freq < 20; freq++){
    radio.setFreq(1400000 + freq * 500);
    displayFreqMode();
  }   */  
// radio.setFreq(1428163);  // set VFO frequency to xx */

  displaySMeter();
  displaySplit_status();
  displayRXTX();
  displayVFO();
  displayFreqMode();
  displayDSP();
  if ((radmode == "CW ") || (radmode == "CWR")) {displayCWConf();}
  else {tft.fillRect(160, 110, 75, 15, TFT_BLUE);
    }
  delay(dly);
}

void displayDSP() {
radio.getAGC_DSP_Conf(AGC,DBF,DNR,DNF);
tft.setCursor(0,110);
tft.setTextSize(2);
if (DBF) {tft.print("DBF");} else {tft.fillRect(0, 110, 35, 15, TFT_BLUE);} // x, y, width, height, color
tft.setCursor(40,110);
if (DNF) {tft.print("DNF");} else {tft.fillRect(40, 110, 35, 15, TFT_BLUE);} // x, y, width, height, color
tft.setCursor(80,110);
if (DNR) {tft.print("DNR");} else {tft.fillRect(80, 110, 35, 15, TFT_BLUE);} // x, y, width, height, color
}

void displayCWConf() {
radio.getCW_MTR_Conf(MeterConf,Keyer,Break_In); 
tft.setTextSize(2);
tft.setCursor(160,110); // (num colonne , num ligne) 
  if (Keyer) {     
     tft.print("KYR");}
     else {tft.fillRect(160, 110, 35, 15, TFT_BLUE);} // x, y, width, height, color
  tft.setCursor(210,110); // (colonne , ligne)
  if (Break_In) { 
    tft.print("BK");}
     else {tft.fillRect(210, 110, 25, 15, TFT_BLUE);} // x, y, width, height, color
}

void displayVFO() { // F6CZV
  VFO = radio.getVFO();
  if (VFO != prevVFO) {
  tft.setTextSize(3);
  tft.fillRect(60, 35, 20, 25, TFT_BLUE); // x, y, width, height, color
  tft.setCursor(60,35); // (num colonne , num ligne) 
  tft.print(VFO); 
  prevVFO = VFO;}
  }

  void displaySMeter() { // F6CZV
  Smeter = radio.getSMeter();
  if (Smeter != prevSmeter) {
  tft.setTextSize(3);
  tft.setCursor(0,0);
  tft.fillRect(0, 0, 100, 25, TFT_BLUE); // x, y, width, height, color
  prevSmeter = Smeter;
  tft.print(Smeter);
  }
  }
  
  void displayRXTX() { // F6CZV
  tft.setTextSize(3);
  On_Air = radio.chkTx();
  if (On_Air) {
    tft.setCursor(200,0);
    tft.fillRect(195, 0, 45, 25, TFT_RED); // x, y, width, height, color
    tft.print("Tx");}
  else
  {tft.setCursor(200,0);
  tft.fillRect(195, 0, 45, 25, TFT_BLUE); // x, y, width, height, color
  tft.print("Rx");
  }
  }

  void displaySplit_status() { // F6CZV
  Split = radio.getSPLIT_status();
  tft.setTextSize(3);
  tft.setCursor(130,0);
  if (Split) {
    tft.print("SPL");}
  else
   {tft.fillRect(130, 0, 55, 25, TFT_BLUE);} // x, y, width, height, color
  }
  
  void displayFreqMode() { // F6CZV
  char frequency[9];
  byte shift = 0;
  unsigned long tempfreq;
  int n;
  tempfreq = radio.getFreqMode();
  sprintf(frequency, "%lu", tempfreq);
  
  if (tempfreq < 10000000 & tempfreq >= 1000000) {
   shift = 1;
   }   
   if (tempfreq < 1000000 & tempfreq >= 100000) {
   shift = 2;
   }
   if (tempfreq < 100000) {
   shift = 3;
   }
  tft.setTextSize(3);
  tft.setCursor(54,75); // (num colonne , num ligne)
  if (shift != 3) {tft.print(".");}
  
  for (n = 0; n < 3;n++) {
    tft.setCursor(70+n*17,75); // (num colonne , num ligne)
    tft.fillRect(70+n*17, 75, 17, 25, TFT_BLUE); // x, y, width, height, color
    tft.print(frequency[3+n-shift]); }
  
  tft.setCursor(120,75); // (num colonne , num ligne)
  tft.print(",");
  
  for (n = 0; n < 2;n++) {
    tft.setCursor(135+n*17,75); // (num colonne , num ligne)
    tft.fillRect(135+n*17, 75, 17, 25, TFT_BLUE); // x, y, width, height, color
    tft.print(frequency[6+n-shift]); }
 
  for (n = 0; n < 3;n++) {
    tft.setCursor(5+n*17,75); // (num colonne , num ligne)
    tft.fillRect(5+n*17, 75, 17, 25, TFT_BLUE); // x, y, width, height, color
    if ((n-shift)>= 0) {tft.print(frequency[n-shift]);} 
  }
  tft.setCursor(177,75); // (num colonne , num ligne)
  tft.print("kHz"); 
  shift = 0 ;
  radmode = radio.getMode();
  if (radmode != prevmode) {
  tft.setTextSize(3);
  tft.setCursor(100,35); // (num colonne , num ligne)
  tft.fillRect(100, 35, 51, 25, TFT_BLUE); // x, y, width, height, color 
  tft.print(radmode);
  prevmode = radmode;}
  }

  

                  
  
